-- initialize player
player = {
            x=1*32, 
            y=1*32, 
            w=16, 
            h=16,
            xvel=0, 
            yvel=0, 
            speed=500, 
            jumpSpeed=450, 
            col=0, 
            len=0, 
            gravity=700, 
            onGround=false,
            isMove=false, 
            collide=false, 
            walljump=false, 
            wallDir="r", 
            wallJumpSpd=275,
            debug=true,
            state="play",
            level="1",
        } 

-- player functions
-------------------

-- states
function player:state_handle()
    if self.state == "menu" then
        return 0 
    elseif self.state == "play" then

    elseif self.state == "dead" then
        return
    end
end

-- keys
function player:jump(key)
    if (key == "up" or key =="or") and self.onGround then
        self.yvel = self
        self.yvel = -self.jumpSpeed
    end
end

function player:wall_jump(key)
    if (key == "up" or key == "w") and self.walljump and not self.onGround then
        self.yvel = 0
        self.yvel = -self.jumpSpeed
        if self.wallDir == "l" then
            self.xvel = self.wallJumpSpd
        elseif self.wallDir == "r" then
            self.xvel = -self.wallJumpSpd
        end
    end
end

function player:quit(key)
    if key == "escape" then
        love.event.quit()
    end
end

function player:toggle_debug(key)
    -- show debug
    if key == "tab" then
        self.debug = not self.debug
    end
    -- reset player positions
    if key == "r" then
        self.reset()
    end
end

function player:reset()
    player.x, player.y = 1*32, 1*32
    player.xvel, player.yvel = 0, 0
    wrld:update(player, player.x, player.y)
end

function player:start()
    if key == "enter" or key == " " and self.state == "menu" then
        self.state = "play"
    end
end


-- update functions 
function player:physics(dt)
    -- gravity
    self.yvel = self.yvel + 2*self.gravity * dt
    if self.yvel > 1.5*self.gravity then self.yvel = self.gravity end
    --friction
    self.xvel = self.xvel * (1 - math.min(dt*2, 1))
end

function player:movement(dt)
    -- movement
    if (love.keyboard.isDown("left") or love.keyboard.isDown("a")) and self.xvel > -self.speed then
        self.xvel = self.xvel - self.speed * dt
        self.isMove = true
    end
    if (love.keyboard.isDown("right") or love.keyboard.isDown("d")) and self.xvel < self.speed then
        self.xvel = self.xvel + self.speed * dt
        self.isMove = true
    end
end

function player:collisions(dt)
    --solid tile collisions
    local nx, ny = self.x + (self.xvel * dt), self.y + (self.yvel * dt)
    local finalX, finalY, collisions = wrld:move(self, nx, ny)

    if #collisions > 0 then
        for i,v in ipairs(collisions) do
            if v.other.type == "solid" then
                if v.normal.y == -1 then
                    self.yvel = 0
                    self.onGround = true
                elseif v.normal.y == 1 then
                   self.yvel = 0
                end
                if v.normal.x == 1 then
                    self.walljump = true
                    self.wallDir = "l"
                    self.xvel = 0
                elseif v.normal.x == -1 then
                    self.walljump = true
                    self.wallDir = "r"
                    self.xvel = 0
                end

            elseif v.other.type == "finish" then
                self.state = "finish"
            end
        end

    -- reset player bool variables
    elseif #collisions < 1 then
        self.walljump = false
        self.onGround = false
    end

    self.x = finalX
    self.y = finalY
end

function player:check_variables()
    -- player onground reset
    if self.onGround then
        if self.yvel ~= 0 then
            self.onGround = false
        end
    end
end


-- draw functions
function player:draw_states()
    if self.state == "play" then end
    
    if self.state == "menu" then
        love.graphics.setColor(0,0,0,128)
        love.graphics.rectangle("fill", 0,0,width*tilewidth,height*tileheight)
        return
    elseif self.state == "dead" then
        love.graphics.setColor(0,0,0,128)
        love.graphics.rectangle("fill", 0,0,love.graphics.getDimensions())
        return
    end
end

function player:draw_debug()
    --debug graphics
    if player.debug then
        love.graphics.setColor(255,0,0)
        local debugStr = "XVEL: " .. math.floor(self.xvel) .. "  " .. "YVEL: " .. math.floor(self.yvel) .. "  " .. "WALLJUMP" .. " " .. tostring(self.walljump)
        love.graphics.print(debugStr, cam:getX()-love.graphics.getWidth()/2+150, cam:getY()-love.graphics.getHeight()/2+100)
    end

    love.graphics.setColor(255,0,0)
    love.graphics.print("Press 'r' to restart", 10*32,2*32)
end


-- MAIN PLAYER FUNCTIONS
------------------------
function player.update(dt)
    -- state handler
    player:state_handle()

    -- regular update functions
    player:movement(dt)
    player:physics(dt)
    player:collisions(dt)
    player:check_variables()
end

function player.draw()
    -- draw states
    player:draw_states()

    -- player graphics
    love.graphics.setColor(0,0,0)
   	love.graphics.rectangle('fill', player.x, player.y, player.w, player.h)
    
    player:draw_debug()    
end

function player.keypressed(key)
    if key == " " and player.state == "menu" then
        player.state = "play"
    end

    -- start
    player:start()

    -- player's regular jump
    player:jump(key)
    -- player's walljump
    player:wall_jump(key)
    -- reset player / show debug
    player:toggle_debug(key)
    -- quit window
    player:quit(key)
    
end